/*jslint nomen: true*/
/*global console, RSVP, renderJS, QUnit, window, document,
 __RenderJSGadget, URL, jIO*/
(function (document, renderJS, QUnit, jIO) {
  "use strict";
  var test = QUnit.test,
    module = QUnit.module,
    RenderJSGadget = __RenderJSGadget,
    jsonform_url = '../jsonform.gadget.html';

  QUnit.config.autostart = false;

  function downloadJSON(url) {
    return RSVP.Queue()
      .push(function () {
        return jIO.util.ajax({
          url: url,
          dataType: "json"
        });
      })
      .push(undefined, function (error) {
        console.log(error);
        throw error;
      })
      .push(function (evt) {
        return evt.target.response;
      });
  }

  function create_gadget(__aq_parent) {
    var gadget = new RenderJSGadget();
    gadget.__sub_gadget_dict = {};
    gadget.__aq_parent = __aq_parent;
    // return gadget.declareGadget(jsonform_url, {
    return gadget.declareGadget((new URL(jsonform_url, window.location)), {
      // sandbox: "iframe",
      element: document.querySelector('#qunit-fixture'),
      scope: 'foobar'
    });
  }

  /////////////////////////////////////////////////////////////////
  // render
  /////////////////////////////////////////////////////////////////
  module("jsonform.general");
  test('JSON Form render', function (assert) {
    var done = assert.async(),
      gadget;
    create_gadget(function (method_name, argument_list) {
      if (method_name === "notifyValid") {
        assert.ok(argument_list, "form correctly filled");
        return "result correctly fetched from parent";
      }
      throw new renderJS.AcquisitionError("Can not handle " + method_name);
    })
      .push(function (g) {
        gadget = g;
        return gadget.render({
          key: "foo"
        });
      })
      .push(function (element) {
        assert.ok(element, "gadget rendered");
      })
      .push(undefined, function (error) {
        assert.notOk(error, "issue in gadget");
      })
      .push(function () {
        done();
      });
  });

  test('JSON Form getContent', function (assert) {
    var done = assert.async(),
      gadget,
      key = "foo_key",
      schema = {
        "properties": {
          "foo": {"type": "integer"},
          "bar": {"type": "string"}
        },
        required: ["foo"]
      },
      value = {foo: 1, bar: "fsdfs"},
      schema_orig = JSON.parse(JSON.stringify(schema));
    create_gadget(function (method_name, argument_list) {
      if (method_name === "notifyValid") {
        assert.ok(argument_list, "form correctly filled");
        return "result correctly fetched from parent";
      }
      throw new renderJS.AcquisitionError("Can not handle " + method_name);
    })
      .push(function (g) {
        gadget = g;
        return gadget.render({
          key: key,
          schema: schema,
          value: value
        });
      })
      .push(function (element) {
        assert.ok(element, "gadget rendered");
        return gadget.getContent();
      })
      .push(function (json_document) {
        assert.deepEqual(schema, schema_orig, "schema not change (side effect absent)");
        assert.deepEqual(JSON.parse(json_document[key]), value, "returned value equal");
      })
      .push(undefined, function (error) {
        assert.notOk(error, "issue in gadget");
      })
      .push(function () {
        done();
      });
  });

  function create_callback(schema, value) {
    return function (assert) {
      var done = assert.async(),
        gadget,
        key = "foo_key",
        schema_orig = JSON.parse(JSON.stringify(schema));
      create_gadget(function (method_name, argument_list) {
        if (method_name === "notifyValid") {
          assert.ok(argument_list, "form correctly filled");
          return "result correctly fetched from parent";
        }
        throw new renderJS.AcquisitionError("Can not handle " + method_name);
      })
        .push(function (g) {
          gadget = g;
          return gadget.render({
            key: key,
            schema: schema,
            value: value
          });
        })
        .push(function (element) {
          assert.ok(element, "gadget rendered");
          return gadget.getContent();
        })
        .push(function (json_document) {
          var returned_value;
          assert.ok(json_document.hasOwnProperty(key), "getContent does not content property");
          assert.deepEqual(schema, schema_orig, "schema changed (side effect on schema)");
          try {
            returned_value = JSON.parse(json_document[key]);
          } catch (error) {
            assert.notOk("value not parsable:'" + returned_value + "'\n" + error);
          }
          assert.deepEqual(returned_value, value,
            "document changed without user interact (side effect on data)");
        })
        .push(undefined, function (error) {
          console.log(assert.test.module.name + ': ' + assert.test.testName);
          console.error(error);
          assert.notOk(error, "issue in gadget");
        })
        .push(function () {
          done();
        });
    };
  }

  RSVP.Queue()
    .push(function () {
      return downloadJSON("../test_index.json");
    })
    .push(function (list) {
      var i,
        tasks = [];
      for (i = 0; i < list.length; i += 1) {
        tasks.push(downloadJSON('../' + list[i]));
      }
      return RSVP.all(tasks);
    })
    .push(function (list) {
      var i,
        k,
        z,
        m,
        t;
      for (i = 0; i < list.length; i += 1) {
        for (k = 0; k < list[i].length; k += 1) {
          m = list[i][k];
          module(m.description);
          for (z = 0; z < m.tests.length; z += 1) {
            t = m.tests[z];
            test(t.description, create_callback(m.schema, t.data, t.valid));
          }
        }
      }
    })
    .push(function () {
      QUnit.start();
    });

}(document, renderJS, QUnit, jIO));